# Makes a playlist file from each album, where album is defined in the
# expression field 'playlist name'.  The playlist is stored in the directory
# of the first file in the view.

package MakePlaylist;

use v5.16;
use strict;
use warnings;
use Encode::Locale;
use Encode;

my $keyfield = 'playlist name';

our %callbacks = (
    perfile => \&do_perfile,
    post    => \&do_postprocessing,
);

my $rows_seen = 0;
my %albums;

#say "ENCODING: ", $Encode::Locale::ENCODING_LOCALE, ';', $Encode::Locale::ENCODING_LOCALE_FS;

# Per-row callback function
sub do_perfile {
    my ($cellval, $row) = @_;

    exists $row->{$keyfield} or
	die "The view is missing the column 'Playlist Name', which defines the playlist file name\nStopped";

    #say 'PerFile: ', $row->{$keyfield}, '; ', $row->{'filename'};
    push @{$albums{$row->{$keyfield}}}, $row->{'filename'};

    $rows_seen++;

    return undef;
}

# Post-processing callback function
sub do_postprocessing {

    #say "Post-processing callback; row_seen = $rows_seen";

    foreach my $key (sort keys %albums) {
	# run through the albums for the list of files to be included in the playlist
	#my ($pathsep) = ($albums{$key}[0] =~ /([\\\/])/);
	#my $filename = ".$pathsep$key.m3u";
	my $filename;
	($filename = $albums{$key}[0]) =~ s/^(.*[\\\/])[^\\\/]+$/$1${key}.m3u/;
	main::verbose 'make playlist file: ', $filename;
	open my $plfile, ">", encode(locale_fs => $filename) or
	    die "Can't create playlist file $filename\n$!\nStopped";

	foreach my $track (sort @{$albums{$key}}) {
	    #$track =~ s/^.*([\\\/][^\\\/]+)/$1/;
	    main::verbose "\tadding Track: .$track";
	    say $plfile $track;
	}
	close $plfile or
	    die "Failed to close playlist file $filename\n$!\nStopped";
    }

    return;
};

1;
